#include <stdio.h>
#include <fcntl.h>
#include <sgtty.h>
#include <signal.h>

struct sgttyb tty,ttysav;
int ttyfd;
char usage[] = {"usage: wcsend filename\n"};

#define FALSE 0
#define TRUE 1

#define DOTS            50      /* Sector counting dots per line */
#define SECSIZ          0x80
#define BufSize         0x1000  /* Text buffer */
#define ERRORMAX        20      /* Max errors before abort */
#define RETRYMAX        15      /* Maximum retrys before abort */
#define TIMEOUT		60     /* Maximum time between xfers */
#define XFERDELAY	30	/* THIS MUST BE < TIMEOUT (time to first try) */

#define SOH             1       /* Start of sector char */
#define EOT             4       /* end of transmission char */
#define ACK             6       /* acknowledge sector transmission */
#define NAK             21      /* error in transmission detected */

static char
    bufr[BufSize];

static int
    fd,
    mtimeout;

static long
    bytes_xferred;

quit()
{
    restoreterm();   /* Restore terminal */
    close(ttyfd);   /* Close terminal */
    fprintf(stderr,"Transfer timed out.\n");
    exit(0);
}


fixterm()
{
    if ((ttyfd = open("/dev/tty",2)) < 0) {   /* Open terminal */
	fprintf(stderr,"\nCouldn't open tty\n");
	exit(0);
    }

    ioctl(ttyfd,TIOCGETP,&tty);   /* Get terminal flags */
    ioctl(ttyfd,TIOCGETP,&ttysav);   /* save a copy */
#ifndef unix
    tty.sg_flags = RAW | TANDEM;
#else
    tty.sg_flags = RAW;
#endif
    ioctl(ttyfd,TIOCSETP,&tty);   /* Save as new characteristics */

    signal(SIGALRM,quit);   /* Catch alarm clock */
    signal(SIGHUP,quit);   /* and hang-up */
    alarm(TIMEOUT);   /* Start the alarm clock */
}

pch(ch)
char ch;
{
    putchar(ch);
    fflush(stdout);
}

int gch()
{
    int cc;

    alarm(TIMEOUT);
    return(getchar());
}

#ifdef FIONREAD
numwait()
{
    int t;

    ioctl(ttyfd,FIONREAD,&t);   /* See if any chars. waiting */
    return(t);
}
#endif

flushinput()
{
#ifdef FIONREAD
    int t;

    do {
	t = numwait();
	if (t > 0)
	    getchar();   /* Yes, toss 'em */
    } while (t > 0);
#else
#define TCFLSH (('T'<<8)|7)
    ioctl(ttyfd, TCFLSH, 0);
#endif
}

restoreterm()
{
    ioctl(ttyfd,TIOCSETP,&ttysav);   /* Restore terminal */
    close(ttyfd);   /* Close terminal */
}

XSend_File(file)
    char *file;
{
    int sectnum, bytes_to_send, size, attempts;
    unsigned checksum, j, bufptr;
    char c;


    if ((fd = open(file, O_RDONLY)) < 0)
        {
        fprintf(stderr,"wcsend: cannot open %s\r\n", file);
        return FALSE;
        }
    else
        fprintf(stderr,"wcsend: sending %s\r\n", file);

    attempts = 0;
    sectnum = 1;
    j = 0;

    flushinput ();

    while (((c = gch ()) != NAK) && (j++ < (ERRORMAX*2)));

    if (j >= (ERRORMAX*2))
        {
        return FALSE;
        };

    while ((bytes_to_send = read(fd, bufr, BufSize)) && attempts != RETRYMAX)
        {
        if (bytes_to_send == EOF)
            {
            close(fd);
            return FALSE;
            };

        bufptr = 0;

        while (bytes_to_send > 0 && attempts != RETRYMAX)
            {
            attempts = 0;

            do
                {
                pch(SOH);
                pch(sectnum);
                pch(~sectnum);
                checksum = 0;
                size = SECSIZ <= bytes_to_send ? SECSIZ : bytes_to_send;
                bytes_to_send -= size;

                for (j = bufptr; j < (bufptr + SECSIZ); j++)
                    if (j < (bufptr + size))
                        {
                        pch(bufr[j]);
                        checksum += bufr[j];
                        }
                    else
                        pch(0);

		fflush(stdout);
                pch(checksum);
                attempts++;
                c = gch();
                }
            while ((c != ACK) && (attempts != RETRYMAX));

            bufptr += size;
            bytes_xferred += size;
            sectnum++;
            }
        }

    close(fd);

    if (attempts == RETRYMAX)
        {
            return FALSE;
        }
    else
        {
        attempts = 0;

        do
            {
            pch(EOT);
            attempts++;
            }
        while ((getchar() != ACK) && (attempts != RETRYMAX));
        };

    return TRUE;
}

main(argc,argv)
int argc;
char *argv[];
{
if (argc != 2) {
	fprintf(stderr,usage);
	exit(1);
	}

	fixterm();
	XSend_File(argv[1]);
	alarm(0);
	restoreterm();
	exit(1);
    }

